import { ClassType, KeyType } from '../utils/types';
import { ClassInfo } from './ClassInfo';
declare type ExtendAndInterface = KeyType;
export interface TypeWithInfo {
    type: ClassType;
    info: ClassInfo;
}
export declare class DuplicateRegistrationError extends Error {
    constructor(type: ClassType);
}
export declare class ClassLoader {
    protected classInfoMap: Map<ClassType, ClassInfo>;
    protected implementCacheMap: Map<ExtendAndInterface, TypeWithInfo[]>;
    constructor(classInfoMap?: Map<ClassType, ClassInfo>, implementCacheMap?: Map<ExtendAndInterface, TypeWithInfo[]>);
    /** has class */
    hasClass(type: ClassType): boolean;
    /** register class */
    registerClass(type: ClassType, info?: ClassInfo): this;
    /** unregister class */
    unregisterClass(type: ClassType): boolean;
    /** get class info */
    getClassInfo(type: ClassType): ClassInfo;
    /** filter classes by info */
    filterClasses(pattern: (info: TypeWithInfo) => boolean): TypeWithInfo[];
    /** classes of extends/implements type */
    getImplementClasses(type: ExtendAndInterface): TypeWithInfo[];
    clearAll(): this;
    /** return new instance from this */
    clone(): ClassLoader;
    /**
     * init maps from instance
     * @param classLoader source loader
     */
    initWith(classLoader: ClassLoader): void;
    protected cloneImplCacheMap(map: Map<KeyType, TypeWithInfo[]>): Map<KeyType, TypeWithInfo[]>;
    protected getImplCacheByType(type: ExtendAndInterface): TypeWithInfo[];
}
export declare const classLoader: ClassLoader;
export {};
